/* BdfServer_JsonProducers - Copyright (c) 2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.jsonproducers.selection;

import net.fichotheque.permission.PermissionSummary;
import java.io.IOException;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.Fiches;
import net.fichotheque.exportation.table.Cell;
import net.fichotheque.exportation.table.CellEngine;
import net.fichotheque.json.CellJson;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.json.JsonProperty2;
import net.mapeadores.util.localisation.Lang;


/**
 *
 * @author Vincent Calame
 */
public class FichesJsonProperty implements JsonProperty2 {

    private final PermissionSummary permissionSummary;
    private final Fiches fiches;
    private final CellEngine cellEngine;
    private final Lang lang;

    public FichesJsonProperty(PermissionSummary permissionSummary, Fiches fiches, CellEngine cellEngine, Lang lang) {
        this.fiches = fiches;
        this.lang = lang;
        this.cellEngine = cellEngine;
        this.permissionSummary = permissionSummary;
    }

    @Override
    public String getName() {
        return "fiches";
    }

    @Override
    public void writeValue(JSONWriter jw) throws IOException {
        jw.object();
        jw.key("count")
                .value(fiches.getFicheCount());
        jw.key("entries");
        jw.array();
        for (Fiches.Entry entry : fiches.getEntryList()) {
            writeEntry(jw, entry);
        }
        jw.endArray();
        jw.endObject();
    }

    private void writeEntry(JSONWriter jw, Fiches.Entry entry) throws IOException {
        Corpus corpus = entry.getCorpus();
        jw.object();
        {
            jw.key("name")
                    .value(corpus.getSubsetName());
            jw.key("title")
                    .value(FichothequeUtils.getTitle(corpus, lang));
            jw.key("array");
            jw.array();
            for (FicheMeta ficheMeta : entry.getFicheMetaList()) {
                writeFiche(jw, ficheMeta);
            }
            jw.endArray();
        }
        jw.endObject();
    }

    private void writeFiche(JSONWriter jw, FicheMeta ficheMeta) throws IOException {
        int ficheid = ficheMeta.getId();
        String corpusName = ficheMeta.getSubsetName();
        Cell[] cellArray = cellEngine.getCellArray(ficheMeta);
        jw.object();
        {
            jw.key("corpus")
                    .value(corpusName);
            jw.key("id")
                    .value(ficheid);
            jw.key("code")
                    .value(corpusName + '/' + ficheid);
            jw.key("title")
                    .value(ficheMeta.getTitre());
            jw.key("editable")
                    .value(permissionSummary.canWrite(ficheMeta));
            CellJson.cellArrayMappingProperty(jw, cellArray);
        }
        jw.endObject();
    }

}
